import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:uuid/uuid.dart';

import 'isolate_bloc_observer.dart';

/// This bloc will work in isolate.
/// [IsolateBlocWrapper<Event, State>] will receive this bloc's state.
/// This bloc must be created from UI with [createBloc<BlocT>] function. But you
/// can use it from another [IsolateBloc].
abstract class IsolateBloc<Event, State> extends Stream<State> implements Sink<State> {
  State state;
  final String id;
  final _stateController = StreamController<State>.broadcast();

  // todo: any errors may contains lambda functions and can't be sended
  void Function(Object) onError;

  /// The current [IsolateBlocObserver].
  static IsolateBlocObserver observer = IsolateBlocObserver();

  IsolateBloc(this.state) : id = Uuid().v4();

  @mustCallSuper
  @override
  void add(Object event) {
    try {
      onEventReceived(event);
    } catch (e) {
      assert(onError != null, 'ON ERROR DOES NOT EXIST!!!!');
      onError(e);
    }
  }

  /// Use this function to send new state to UI.
  @mustCallSuper
  void emit(State state) {
    _stateController.add(state);
  }

  StreamSubscription<State> listen(
    void onData(State event), {
    Function onError,
    void onDone(),
    bool cancelOnError,
  }) {
    return _prepareStateStream.listen(
      onData,
      onError: onError,
      onDone: onDone,
      cancelOnError: cancelOnError,
    );
  }

  Stream<State> get _prepareStateStream async* {
    yield state;
    yield* _stateController.stream;
  }

  /// This function receive events from UI.
  void onEventReceived(Event event);

  Stream<State> mapEventToState(Event event) async* {}

  @mustCallSuper
  Future<void> close() async {
    await _stateController.close();
  }
}
