import 'isolate_bloc.dart';

typedef IsolateBlocCreator<Event, State> = IsolateBloc<Event, State> Function();

/// Maintain [IsolateBloc]s in isolate
class IsolateBlocConnector {
  static final IsolateBlocConnector instance = IsolateBlocConnector._();
  final _initialStates = <Type, Object>{};
  final _freeBlocs = <Type, IsolateBloc>{};
  final _blocCreators = <Type, IsolateBlocCreator>{};
  final _createdBlocs = <String, IsolateBloc>{};

  Map<Type, Object> get initialStates => _initialStates;

  IsolateBlocConnector._();

  /// Reterns new bloc from cached in [_freeBlocs] or create new one.
  IsolateBloc getFreeBlocByType(Type type) {
    IsolateBloc bloc;
    if (_freeBlocs.containsKey(type)) {
      bloc = _freeBlocs.remove(type);
    }
    bloc = _blocCreators[type]();
    _createdBlocs[bloc.id] = bloc;
    return bloc;
  }

  IsolateBloc getBlocById(String id) {
    return _createdBlocs[id];
  }

  void register<Event, State>(IsolateBlocCreator<Event, State> creator) {
    // ignore: close_sinks
    final bloc = creator();
    _initialStates[bloc.runtimeType] = bloc.state;
    _blocCreators[bloc.runtimeType] = creator;
  }
}
