import 'dart:async';

import 'bloc_manager.dart';

import 'service_events.dart';

/// Listen for [ServiceEvent]s from isolate
class ServiceEventListener {
  final void Function(ServiceEvent) sendEvent;
  final Stream<ServiceEvent> _eventsStream;
  final _initializeClompleter = Completer<Map<Type, Object>>();

  ServiceEventListener(this.sendEvent, this._eventsStream) {
    _eventsStream.listen(listener);
  }

  Future<Map<Type, Object>> getInitialStates() async {
    return _initializeClompleter.future;
  }

  void listener(ServiceEvent event) {
    if (event is IsolateBlocsInitialized) {
      _initializeClompleter.complete(event.initialStates);
    } else if (event is IsolateBlocCreatedEvent) {
      BlocManager.instance.bindFreeWrapper(event.blocType, event.blocUuid);
    } else if (event is IsolateBlocTransitionEvent) {
      BlocManager.instance.getWrapper(event.blocUuid).stateReceiver(event.event);
    } else if (event is IsolateBlocErrorEvent) {
      BlocManager.instance.getWrapper(event.blocUuid).onError(event.error);
    }
  }
}
